@@ -104,8 +104,33 @@ class DamonRecord:
         ordered_dict['snapshots'] = [s.to_kvpairs(raw) for s in self.snapshots]
         return ordered_dict
 
+    def can_merge(self, other):
+        return (
+                self.kdamond_idx == other.kdamond_idx and
+                self.context_idx == other.context_idx and
+                self.intervals == other.intervals and
+                self.scheme_idx == other.scheme_idx and
+                self.target_id == other.target_id)
+
+    def merge(self, other):
+        self.snapshots += other.snapshots
+        self.snapshots.sort(key=lambda s: s.start_time)
+
 # for monitoring results manipulation
 
+def merge_records(records):
+    merged_records = []
+    for record in records:
+        merged = False
+        for merged_record in merged_records:
+            if merged_record.can_merge(record):
+                merged_record.merge(record)
+                merged = True
+                break
+        if not merged:
+            merged_records.append(record)
+    return merged_records
+
 def regions_intersect(r1, r2):
     return not (r1.end <= r2.start or r2.end <= r1.start)
 
@@ -903,14 +928,21 @@ def poll_target_pids(kdamonds):
     return False
 
 class RecordingHandle:
-    # for tracepoint recording
-    tracepoint = None
     file_path = None
     file_format = None
     file_permission = None
+
+    # for access patterns tracing
+    tracepoint = None
     monitoring_intervals = None
     perf_pipe = None
 
+    # for access patterns snapshot
+    snapshot_request = None
+    snapshot_records = None
+    snapshot_count = None
+    snapshot_interval_sec = None
+
     # for CPU clock event recording
     do_profile = None
     perf_profile_pipe = None
@@ -962,6 +994,11 @@ def start_recording(handle):
         handle.perf_profile_pipe = subprocess.Popen(cmd)
 
     start_time = time.time()
+    nr_snapshots_to_take = handle.snapshot_count
+    if handle.snapshot_interval_sec:
+        sleep_time = handle.snapshot_interval_sec
+    else:
+        sleep_time = 1
     while (poll_target_pids(handle.kdamonds) or
            _damon.any_kdamond_running()):
         if handle.add_child_tasks is True:
@@ -977,24 +1014,45 @@ def start_recording(handle):
         if (handle.timeout is not None and
             time.time() - start_time >= handle.timeout):
             break
-        time.sleep(1)
+
+        if handle.snapshot_request:
+            if handle.snapshot_records is None:
+                handle.snapshot_records = []
+            snapshot_records, err = get_snapshot_records_of(
+                    handle.snapshot_request)
+            if err is not None:
+                print('failed getting snapshot')
+                exit(1)
+            handle.snapshot_records += snapshot_records
+            nr_snapshots_to_take -= 1
+            if nr_snapshots_to_take == 0:
+                handle.snapshot_records = merge_records(
+                        handle.snapshot_records)
+                break
+
+        time.sleep(sleep_time)
 
 def finish_recording(handle):
-    try:
-        handle.perf_pipe.send_signal(signal.SIGINT)
-        handle.perf_pipe.wait()
-    except:
-        # perf might already finished
-        pass
+    if handle.perf_pipe:
+        try:
+            handle.perf_pipe.send_signal(signal.SIGINT)
+            handle.perf_pipe.wait()
+        except:
+            # perf might already finished
+            pass
 
-    if handle.file_format == file_type_perf_data:
-        os.chmod(handle.file_path, handle.file_permission)
-    else:
-        err = update_records_file(handle.file_path, handle.file_format,
-                handle.file_permission, handle.monitoring_intervals)
-        if err != None:
-            print('converting format from perf_data to %s failed (%s)' %
-                    (handle.file_format, err))
+        if handle.file_format == file_type_perf_data:
+            os.chmod(handle.file_path, handle.file_permission)
+        else:
+            err = update_records_file(handle.file_path, handle.file_format,
+                    handle.file_permission, handle.monitoring_intervals)
+            if err != None:
+                print('converting format from perf_data to %s failed (%s)' %
+                        (handle.file_format, err))
+
+    if handle.snapshot_records:
+        write_damon_records(handle.snapshot_records, handle.file_path,
+                            handle.file_format, handle.file_permission)
 
     if handle.perf_profile_pipe is not None:
         try:
