@@ -225,6 +225,32 @@ def damos_options_to_scheme(sz_region, access_rate, age, action,
     except Exception as e:
         return None, 'Wrong \'--damos_*\' argument (%s)' % e
 
+def convert_add_damos_filter_out_args_to_damos_filter_args(args):
+    '''
+    Convert --damos_filter_out arguments to --damos_filter arguments form, and
+    append to --damos_filter so that existing --damos_filter handling code can
+    just reused.
+
+    --damos_filter_out receives three arguments: type, matching, and optional
+    arguments for the type of filter (memcg path for memcg type).  Those are
+    same to --damos_filter, but 'matching' is optional.  The default value is
+    'matching'.
+    '''
+    for filter_out_args in args.damos_filter_out:
+        if len(filter_out_args) < 1:
+            return '--damos_filter_out with no argument'
+        if filter_out_args[0] == 'not':
+            fmatching = 'nomatching'
+            filter_out_args = filter_out_args[1:]
+            if len(filter_out_args) < 1:
+                return 'filter type not given'
+        else:
+            fmatching = 'matching'
+        ftype = filter_out_args[0]
+        ftype_args = filter_out_args[1:]
+        args.damos_filter.append([ftype, fmatching] + ftype_args)
+    return None
+
 def damos_options_to_schemes(args):
     if args.damos_quota_interval:
         for i, interval in enumerate(args.damos_quota_interval):
@@ -258,6 +284,10 @@ def damos_options_to_schemes(args):
         return [], 'wrong --damos_nr_quota_goals'
     if len(args.damos_wmarks) > nr_schemes:
         return [], 'too much --damos_wmarks'
+    if len(args.damos_filter_out) > 0:
+        err = convert_add_damos_filter_out_args_to_damos_filter_args(args)
+        if err is not None:
+            return [], 'converting damos_filter_out failed (%s)' % err
     # for multiple schemes, number of filters per scheme is required
     if len(args.damos_filter) > 0 and nr_schemes > 1:
         if len(args.damos_nr_filters) == 0:
@@ -537,102 +567,149 @@ def set_common_argparser(parser):
     parser.add_argument('--debug_damon', action='store_true',
             help='Print debugging log')
 
-def set_monitoring_attrs_pinpoint_argparser(parser):
+def set_monitoring_attrs_pinpoint_argparser(parser, hide_help=False):
     # for easier pinpoint setup
-    parser.add_argument('-s', '--sample', metavar='<microseconds>',
-            default=5000, help='sampling interval (us)')
-    parser.add_argument('-a', '--aggr', metavar='<microseconds>',
-            default=100000, help='aggregate interval (us)')
-    parser.add_argument('-u', '--updr', metavar='<microseconds>',
-            default=1000000, help='regions update interval (us)')
-    parser.add_argument('-n', '--minr', metavar='<# regions>',
-            default=10, help='minimal number of regions')
-    parser.add_argument('-m', '--maxr', metavar='<# regions>',
-            default=1000, help='maximum number of regions')
+    parser.add_argument(
+            '-s', '--sample', metavar='<microseconds>',
+            default=5000, help='sampling interval (us)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '-a', '--aggr', metavar='<microseconds>',
+            default=100000, help='aggregate interval (us)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '-u', '--updr', metavar='<microseconds>',
+            default=1000000, help='regions update interval (us)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '-n', '--minr', metavar='<# regions>',
+            default=10, help='minimal number of regions'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '-m', '--maxr', metavar='<# regions>',
+            default=1000, help='maximum number of regions'
+            if not hide_help else argparse.SUPPRESS)
 
-def set_monitoring_attrs_argparser(parser):
+def set_monitoring_attrs_argparser(parser, hide_help=False):
     # for easier total setup
     parser.add_argument('--monitoring_intervals', nargs=3,
-            default=['5ms', '100ms', '1s'],
-            metavar=('<sample>', '<aggr>', '<update>'),
-            help='monitoring intervals (us)')
+                        default=['5ms', '100ms', '1s'],
+                        metavar=('<sample>', '<aggr>', '<update>'),
+                        help='monitoring intervals (us)'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--monitoring_nr_regions_range', nargs=2,
-            metavar=('<min>', '<max>'), default=[10, 1000],
-            help='min/max number of monitoring regions')
+                        metavar=('<min>', '<max>'), default=[10, 1000],
+                        help='min/max number of monitoring regions'
+                        if not hide_help else argparse.SUPPRESS)
 
-def set_monitoring_argparser(parser):
+def set_monitoring_argparser(parser, hide_help=False):
     parser.add_argument('--ops', choices=['vaddr', 'paddr', 'fvaddr'],
-            help='monitoring operations set')
+                        help='monitoring operations set'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--target_pid', type=int, metavar='<pid>',
-            help='pid of monitoring target process')
+                        help='pid of monitoring target process'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('-r', '--regions', metavar='"<start>-<end> ..."',
-            type=str, default='', help='monitoring target address regions')
-    parser.add_argument('--numa_node', metavar='<node id>', type=int,
-            help='if target is \'paddr\', limit it to the numa node')
-    set_monitoring_attrs_argparser(parser)
+                        type=str, default='',
+                        help='monitoring target address regions'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--numa_node', metavar='<node id>', type=int,
+            help='if target is \'paddr\', limit it to the numa node'
+            if not hide_help else argparse.SUPPRESS)
+    set_monitoring_attrs_argparser(parser, hide_help)
 
-def set_damos_argparser(parser):
+def set_damos_argparser(parser, hide_help):
     parser.add_argument('--damos_action', metavar='<action>', nargs='+',
-            action='append', default=[],
-            help='damos action to apply to the target regions')
+                        action='append', default=[],
+                        help='damos action to apply to the target regions'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_sz_region', metavar=('<min>', '<max>'),
+                        nargs=2, default=[], action='append',
+                        help='min/max size of damos target regions (bytes)'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_access_rate', metavar=('<min>', '<max>'),
             nargs=2, default=[], action='append',
-            help='min/max size of damos target regions (bytes)')
-    parser.add_argument('--damos_access_rate', metavar=('<min>', '<max>'),
-            nargs=2, default=[], action='append',
-            help='min/max access rate of damos target regions (percent)')
-    parser.add_argument('--damos_age', metavar=('<min>', '<max>'), nargs=2,
+            help='min/max access rate of damos target regions (percent)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_age', metavar=('<min>', '<max>'), nargs=2,
             default=[], action='append',
-            help='min/max age of damos target regions (microseconds)')
+            help='min/max age of damos target regions (microseconds)'
+            if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_apply_interval', metavar='<microseconds>',
-            action='append', default=[],
-            help='the apply interval for the scheme')
-    parser.add_argument('--damos_quotas', default=[],
+                        action='append', default=[],
+                        help='the apply interval for the scheme'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_quotas', default=[],
             metavar='<quota parameter>', nargs='+', action='append',
             help=' '.join([
             'damos quotas (<time (ms)> [<size (bytes)> [<reset interval (ms)>',
                 '[<size priority weight (permil)>',
                 '[<access rate priority weight> (permil)',
-                '[<age priority weight> (permil)]]]]])']))
+                '[<age priority weight> (permil)]]]]])'])
+            if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_quota_interval', default=[],
                         metavar='<milliseconds>', action='append',
-                        help='quota reset interval')
+                        help='quota reset interval'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_quota_time', default=[],
                         metavar='<milliseconds>', action='append',
-                        help='time quota')
+                        help='time quota'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_quota_space', default=[], metavar='<bytes>',
-                        action='append', help='space quota')
+                        action='append', help='space quota'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--damos_quota_weights', default=[],
                         metavar='<permil>', nargs=3, action='append',
-                        help='quota\'s prioritization weights')
-    parser.add_argument('--damos_quota_goal', nargs='+', action='append',
+                        help='quota\'s prioritization weights'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_quota_goal', nargs='+', action='append',
             default=[],
             metavar='<metric or value>',
             help=' '.join([
                 'damos quota goal (<metric> <target value> [current value]).',
                 '<metric> should be {%s}.' %
-                ','.join(_damon.qgoal_metrics)]))
-    parser.add_argument('--damos_nr_quota_goals', type=int, nargs='+',
+                ','.join(_damon.qgoal_metrics)])
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_nr_quota_goals', type=int, nargs='+',
             default=[], metavar='<integer>',
-            help='number of quota goals for each scheme (in order)')
-    parser.add_argument('--damos_wmarks', nargs=5, action='append', default=[],
+            help='number of quota goals for each scheme (in order)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_wmarks', nargs=5, action='append', default=[],
             metavar=('<metric (none|free_mem_rate)>', '<interval (us)>',
                 '<high mark (permil)>', '<mid mark (permil)>',
                 '<low mark (permil)>'),
-            help='damos watermarks')
-    parser.add_argument('--damos_filter', nargs='+', action='append',
+            help='damos watermarks'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_filter', nargs='+', action='append',
             default=[],
             metavar='<filter argument>',
-            help='damos filter (type, matching, and optional arguments')
-    parser.add_argument('--damos_nr_filters', type=int, nargs='+', default=[],
+            help='damos filter (type, matching, and optional arguments)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_filter_out', nargs='+', action='append',
+            default=[],
+            metavar='<filter argument>',
+            help='damos filter (matching, type, and optional arguments)'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--damos_nr_filters', type=int, nargs='+', default=[],
             metavar='<integer>',
-            help='number of filters for each scheme (in order)')
+            help='number of filters for each scheme (in order)'
+            if not hide_help else argparse.SUPPRESS)
 
-def set_argparser(parser, add_record_options):
+def set_argparser(parser, add_record_options, min_help=True):
     if parser == None:
         parser = argparse.ArgumentParser()
-    set_monitoring_argparser(parser)
-    set_damos_argparser(parser)
+    set_monitoring_argparser(parser, min_help)
+    set_damos_argparser(parser, min_help)
     parser.add_argument('-c', '--schemes', metavar='<json string or file>',
 	    help='data access monitoring-based operation schemes')
     parser.add_argument('--kdamonds', metavar='<json/yaml string or file>',
@@ -650,5 +727,11 @@ def set_argparser(parser, add_record_options):
         parser.add_argument('-o', '--out', metavar='<file path>', type=str,
                 default='damon.data', help='output file path')
     set_common_argparser(parser)
-    set_monitoring_attrs_pinpoint_argparser(parser)
+    set_monitoring_attrs_pinpoint_argparser(parser, min_help)
+    if min_help:
+        if parser.epilog is None:
+            parser.epilog = ''
+        parser.epilog += ' '.join([
+            "DAMON parameters options from 'damo args damon' are also",
+            "supported. Do 'damo args damon -h' for the options."])
     return parser
