@@ -69,6 +69,43 @@ def handle_args(args):
         print(err)
         exit(-3)
 
+def mk_handle(args, kdamonds, monitoring_intervals):
+    if args.schemes_target_regions == False:
+        tracepoint = _damo_records.perf_event_damon_aggregated
+    else:
+        tracepoint = _damo_records.perf_event_damos_before_apply
+
+    handle = _damo_records.RecordingHandle(
+            # for access pattern monitoring
+            tracepoint=tracepoint, file_path=args.out,
+            file_format=args.output_type,
+            file_permission=args.output_permission,
+            monitoring_intervals=monitoring_intervals,
+            # for perf profile
+            do_profile=args.profile is True,
+            # for children processes recording and memory footprint
+            kdamonds=kdamonds, add_child_tasks=args.include_child_tasks,
+            record_mem_footprint=args.footprint,
+            record_vmas=args.vmas, timeout=None)
+
+    if args.snapshot is not None:
+        handle.tracepoint = None
+        tried_regions_of = None
+        if args.schemes_target_regions:
+            tried_regions_of = []
+            for kidx, kd in enumerate(kdamonds):
+                for cidx, ctx in enumerate(kd.contexts):
+                    for sidx, scheme in enumerate(ctx.schemes):
+                        tried_regions_of.append([kidx, cidx, sidx])
+        handle.snapshot_request = _damo_records.RecordGetRequest(
+                tried_regions_of=tried_regions_of, record_file=None,
+                record_filter=None, total_sz_only=False,
+                dont_merge_regions=False)
+        handle.snapshot_interval_sec = args.snapshot[0]
+        handle.snapshot_count = args.snapshot[1]
+
+    return handle
+
 def main(args):
     global data_for_cleanup
 
@@ -106,23 +143,8 @@ def main(args):
                 0].contexts[0].intervals
         kdamonds = data_for_cleanup.orig_kdamonds
 
-    if args.schemes_target_regions == False:
-        tracepoint = _damo_records.perf_event_damon_aggregated
-    else:
-        tracepoint = _damo_records.perf_event_damos_before_apply
+    record_handle = mk_handle(args, kdamonds, monitoring_intervals)
 
-    record_handle = _damo_records.RecordingHandle(
-            # for access pattern monitoring
-            tracepoint=tracepoint, file_path=args.out,
-            file_format=args.output_type,
-            file_permission=args.output_permission,
-            monitoring_intervals=monitoring_intervals,
-            # for perf profile
-            do_profile=args.profile is True,
-            # for children processes recording and memory footprint
-            kdamonds=kdamonds, add_child_tasks=args.include_child_tasks,
-            record_mem_footprint=args.footprint,
-            record_vmas=args.vmas, timeout=None)
     data_for_cleanup.record_handle = record_handle
 
     print('Press Ctrl+C to stop')
@@ -148,16 +170,12 @@ def set_argparser(parser):
                         help='record schemes tried to be applied regions')
     parser.add_argument('--no_profile', action='store_false', dest='profile',
                         help='do not record profiling information')
-    parser.add_argument('--profile', action='store_true',
-                        help='record profiling information together')
     parser.add_argument('--no_footprint', action='store_false',
                         dest='footprint',
                         help='do not record memory footprint')
-    parser.add_argument('--footprint', action='store_true',
-                        help='record memory footprint')
     parser.add_argument('--no_vmas', action='store_false', dest='vmas',
                         help='record virtual memory areas (/proc/<pid>/maps)')
-    parser.add_argument('--vmas', action='store_true',
-                        help='record virtual memory areas (/proc/<pid>/maps)')
+    parser.add_argument('--snapshot', metavar=('<delay>', '<count>'), nargs=2,
+                        type=float, help='record accesses as snapshots')
     parser.description = 'Record monitoring results'
     return parser
